@@ -1,7 +1,7 @@
 # ABSTRACT: Utilities for Monitoring ElasticSearch
 package App::ElasticSearch::Utilities;
 
-our $VERSION = '4.3'; # VERSION
+our $VERSION = '4.4'; # VERSION
 
 use strict;
 use warnings;
@@ -18,9 +18,13 @@ our @_CONFIGS = (
 use Elastijk;
 use CLI::Helpers qw(:all);
 use Getopt::Long qw(:config pass_through);
+use Hash::Flatten qw(flatten);
 use Hash::Merge::Simple qw(clone_merge);
+use Hijk;
 use JSON::XS;
 use Time::Local;
+use URI;
+use URI::QueryParam;
 use YAML;
 
 use Sub::Exporter -setup => {
@@ -33,6 +37,7 @@ use Sub::Exporter -setup => {
         es_indices
         es_indices_meta
         es_index_valid
+        es_index_bases
         es_index_strip_date
         es_index_days_old
         es_index_shards
@@ -52,7 +57,7 @@ use Sub::Exporter -setup => {
         config  => [qw(es_globals)],
         default => [qw(es_connect es_indices es_request)],
         indices => [qw(:default es_indices_meta)],
-        index   => [qw(:default es_index_valid es_index_fields es_index_days_old)],
+        index   => [qw(:default es_index_valid es_index_fields es_index_days_old es_index_bases)],
     },
 };
 use App::ElasticSearch::Utilities::VersionHacks qw(_fix_version_request);
@@ -260,17 +265,45 @@ sub es_request {
         output({color=>'cyan'}, "Called es_request($index / $options->{command}), but --noop and method is $options->{method}");
         return;
     }
-    eval {
-        debug("calling es_request($index / $options->{command})");
-        ($status,$res) = Elastijk::request($options);
-    };
-    my $err = $@;
-    if( $err || !defined $res ) {
-        output({color=>'red',stderr=>1}, "es_request($index / $options->{command}) failed[$status]: $err");
+    # _cat Queries don't work with Elastijk
+    if( $url =~ m#^_cat/# ) {
+        my $uri = URI->new(sprintf "http://%s:%d/%s", $options->{host}, $options->{port}, $url);
+        # Merge URI keys and uri_param keys
+        if ( exists $options->{uri_param} ) {
+            foreach my $k (keys %{ $options->{uri_param} }) {
+                $uri->query_param( $k => $options->{uri_param}{$k} );
+            }
+        }
+        eval {
+            my $resp = Hijk::request({
+                host            => $uri->host,
+                port            => $uri->port,
+                method          => 'GET',
+                path            => $uri->path,
+                query_string    => $uri->query,
+                connect_timeout => $DEF{timeout},
+                read_timeout    => $DEF{timeout},
+            });
+            $status = $resp->{status};
+            die sprintf "%d - %s", $resp->{error_number}, $resp->{error_string} unless $status == 200;
+            $res = [ map { s/^\s+//; s/\s+$//; $_ } grep { defined && length } split /\r?\n/, $resp->{body} ];
+            1;
+        } or do {
+            output({color=>'red',stderr=>1},sprintf "es_request() hijk::request(%s?%s) failed: %s", $uri->path, $uri->query, $@);
+        };
     }
-    elsif($status != 200) {
-        verbose({color=>'yellow'},"es_request($index / $options->{command}) returned HTTP Status $status");
+    else {
+        eval {
+            debug("calling es_request($index / $options->{command})");
+            ($status,$res) = Elastijk::request($options);
+            die sprintf "[%d] Empty response from %s:%d", $status, $options->{host}, $options->{port} unless defined $res;
+            1;
+        } or do {
+            my $err = $@;
+            output({color=>'red',stderr=>1}, "es_request($index / $options->{command}) failed[$status]: $err");
+        };
     }
+    verbose({color=>'yellow'},"es_request($index / $options->{command}) returned HTTP Status $status") if $status != 200;
 
     return $res;
 }
@@ -329,24 +362,24 @@ sub es_indices {
         push @indices, $DEF{INDEX} if es_index_valid( $DEF{INDEX} );
     }
     else {
-        my %meta = es_indices_meta();
-        foreach my $index (sort keys %meta) {
+        my $res = es_request('_cat/indices?h=index,status');
+        foreach my $line (@{ $res }) {
+            my ($index,$status) = split /\s+/, $line;
             debug("Evaluating '$index'");
             if(!exists $args{_all}) {
                 # State Check Disqualification
                 if($args{state} ne 'all'  && $args{check_state})  {
-                    my $result = $meta{$index}->{state} eq $args{state};
+                    my $result = $status eq $args{state};
                     debug({indent=>1,color=>$result ? 'green' : 'red' },
-                        sprintf('+ method:state=%s, got %s', $args{state}, $meta{$index}->{state})
+                        sprintf('+ method:state=%s, got %s', $args{state}, $status)
                     );
                     next unless $result;
                 }
 
                 if( defined $DEF{BASE} ) {
                     debug({indent=>1}, "+ method:base - $DEF{BASE}");
-                    my @parts = split /[\-_]/, $index;
-                    my %parts = map { lc($_) => 1 } @parts;
-                    next unless exists $parts{$DEF{BASE}};
+                    my %bases = map { $_ => 1 } es_index_bases($index);
+                    next unless exists $bases{$DEF{BASE}};
                 }
                 else {
                     my $p = es_pattern;
@@ -358,8 +391,8 @@ sub es_indices {
                     debug({indent=>2,color=>"yellow"}, "+ checking to see if index is in the past $DEF{DAYS} days.");
 
                     my $days_old = es_index_days_old( $index );
-                    debug("$index is $days_old days old");
-                    if( $days_old < 0 ) {
+                    debug(sprintf "%s is %s days old", $index, defined $days_old ? $days_old : 'undef');
+                    if( !defined $days_old ) {
                         debug({indent=>2,color=>'red'}, "! error locating date in string, skipping !");
                         next;
                     }
@@ -388,18 +421,52 @@ sub es_index_strip_date {
 
     return -1 unless defined $index;
 
-    if( my ($indexName) = ($index =~ /^(.*)[-_]$PATTERN_REGEX{DATE}$/) ) {
-        return $indexName;
+    if( $index =~ s/[-_]$PATTERN_REGEX{DATE}// ) {
+        return $index;
     }
     return undef;
 }
 
 
+my %_stripped=();
+
+sub es_index_bases {
+    my ($index) = @_;
+
+    return unless defined $index;
+
+    # Strip to the base
+    my $stripped = es_index_strip_date($index);
+    return unless defined $stripped and length $stripped;
+
+    # Compute if we haven't already memoized
+    if( !exists $_stripped{$stripped} ) {
+        my %bases=();
+        my @parts = grep { defined && length } split /[-_]/, $stripped;
+        debug(sprintf "es_index_bases(%s) dissected to %s", $index, join(',', @parts));
+        my $sep = index( $stripped, '_' ) >= 0 ? '_' : '-';
+
+        my %collected = ();
+        while( my $word = shift @parts ) {
+            my @set=($word);
+            $collected{$word} =1;
+            foreach my $sub ( @parts ) {
+                push @set, $sub;
+                $collected{join($sep,@set)} =1;
+            }
+        }
+        $_stripped{$stripped} = [ sort keys %collected ]
+    }
+
+    return @{ $_stripped{$stripped} };
+}
+
+
 my $NOW = timelocal(0,0,0,(localtime)[3,4,5]);
 sub es_index_days_old {
     my ($index) = @_;
 
-    return -1 unless defined $index;
+    return unless defined $index;
 
     if( my ($dateStr) = ($index =~ /($PATTERN_REGEX{DATE})/) ) {
         my @date=();
@@ -416,7 +483,7 @@ sub es_index_days_old {
         my $diff = $NOW - $idx_time;
         return int($diff / 86400);
     }
-    return -1;
+    return;
 }
 
 
@@ -462,25 +529,43 @@ sub es_index_fields {
     # Handle Version incompatibilities
     my $ref = exists $result->{$index}{mappings} ? $result->{$index}{mappings} : $result->{$index};
 
-    my %fields = ();
-
     # Loop through the mappings, skipping _default_
     my @mappings = grep { $_ ne '_default_' } keys %{ $ref };
+    my %f = ();
     foreach my $mapping (@mappings) {
         next unless exists $ref->{$mapping}{properties};
-        @fields{_extract_fields($ref->{$mapping}{properties})} = ();
+        my $f = \%f;
+
+        foreach my $k (keys %{ $ref->{$mapping}{properties} }) {
+            _flatten_mapping_hash($f, $k, $ref->{$mapping}{properties}{$k});
+        }
+    }
+    my $flat = flatten(\%f, {HashDelimiter=>'.', ArrayDelimiter=>'.'});
+    my %uniq = ();
+    foreach my $k (keys %{ $flat }) {
+        my $u = (split /\./, $k)[-1];
+        $uniq{$u} ||= 0;
+        $uniq{$u}++;
+    }
+    my %fields = %{ $flat };
+    foreach my $k (grep { $uniq{$_} == 1 } keys %uniq) {
+        $fields{$k} = 1;
     }
 
     return wantarray ? keys %fields : [ keys %fields ];
 }
 
-sub _extract_fields {
-    my $ref = shift;
-    my @fields = ();
-    foreach my $key ( keys %{$ref} ) {
-        push @fields, exists $ref->{$key}{properties} ? _extract_fields( $ref->{$key}{properties}, $key ) : $key;
+sub _flatten_mapping_hash {
+    my ($f,$k,$ref) = @_;
+
+    $f->{$k} ||= 1;
+
+    if( exists $ref->{properties} ) {
+        $f->{$k} = {};
+        foreach my $sk (keys %{ $ref->{properties} }) {
+            _flatten_mapping_hash($f->{$k},$sk, $ref->{properties}{$sk});
+        }
     }
-    return @fields;
 }
 
 
@@ -613,7 +698,7 @@ App::ElasticSearch::Utilities - Utilities for Monitoring ElasticSearch
 
 =head1 VERSION
 
-version 4.3
+version 4.4
 
 =head1 SYNOPSIS
 
@@ -673,7 +758,7 @@ Returns a hashref of the pattern filter used to get the indexes
 Without options, this connects to the server defined in the args.  If passed
 an array ref, it will use that as the connection definition.
 
-=head2 es_request([$handle],$command,{ method => 'GET', parameters => { a => 1 } }, {})
+=head2 es_request([$handle],$command,{ method => 'GET', uri_param => { a => 1 } }, {})
 
 Retrieve URL from ElasticSearch, returns a hash reference
 
@@ -727,6 +812,10 @@ Default is 1, set to 0 to disable checking index age.
 
 Returns the index name with the date removed.
 
+=head2 es_index_bases( 'index-name' )
+
+Returns an array of the possible index base names for this index
+
 =head2 es_index_days_old( 'index-name' )
 
 Return the number of days old this index is.
