@@ -9,6 +9,7 @@ use App::ElasticSearch::Utilities qw(:default :index);
 use App::ElasticSearch::Utilities::Query;
 use App::ElasticSearch::Utilities::QueryString;
 use CLI::Helpers qw(:all);
+use File::Basename;
 use File::Slurp::Tiny qw(read_lines);
 use Getopt::Long qw(:config posix_default no_ignore_case no_ignore_case_always);
 use Hash::Merge::Simple qw(clone_merge);
@@ -25,7 +26,7 @@ GetOptions(\%OPT, qw(
     source=s
     destination:s
     append|A
-    block:1000
+    block:i
     mapping:s
     settings:s
     help|h
@@ -36,12 +37,14 @@ GetOptions(\%OPT, qw(
 # Documentation
 pod2usage(1) if $OPT{help};
 pod2usage(-exitstatus => 0, -verbose => 2) if $OPT{manual};
+debug_var(\%OPT);
 
 #------------------------------------------------------------------------#
 # Copy To/From
 my %INDEX = (
     from => $OPT{source},
     to   => exists $OPT{destination} ? $OPT{destination} : $OPT{source},
+    block => exists $OPT{block} ? $OPT{block} : 1000,
 );
 my %HOST = (
     from => $OPT{from},
@@ -62,12 +65,12 @@ my $q  = @ARGV ? $qs->expand_query_string(@ARGV)
                : App::ElasticSearch::Utilities::Query->new(must => {match_all=>{}});
 
 $q->set_scan_scroll('1m');
-$q->set_size( $OPT{'block'} );
+$q->set_size( $INDEX{block} );
 
 # Connect to ElasticSearch
 my %ES = ();
 foreach my $dir (qw(from to)) {
-    $ES{$dir} = es_connect( [ "$OPT{$dir}:9200" ] );
+    $ES{$dir} = es_connect( [ "$HOST{$dir}:9200" ] );
 }
 
 die "Invalid index: $INDEX{from}" unless $ES{from}->exists( index => $INDEX{from} );
@@ -80,7 +83,7 @@ my ($status, $res);
 
 # Mappings/Settings for Non-existant index.
 unless( exists $OPT{append} ) {
-    die "Index $INDEX{to} already exists in $OPT{to}" if $TO_EXISTS;
+    die "Index $INDEX{to} already exists in $HOST{to}" if $TO_EXISTS;
     $res = es_request($ES{from}, '_settings', {index => $INDEX{from}} );
     debug_var($res);
     my $from_settings = $res->{$INDEX{from}}{settings};
@@ -127,7 +130,7 @@ unless( exists $OPT{append} ) {
     );
 
     if ($status ne "200") {
-        die "Failed to create index in $OPT{to} (http status = $status): " . $JSON->encode([ $status, $res ]);
+        die "Failed to create index in $HOST{to} (http status = $status): " . $JSON->encode([ $status, $res ]);
     }
 }
 else {
@@ -135,7 +138,7 @@ else {
     foreach my $k (qw(settings mapping)) {
         push @ignored, $k if exists $OPT{$k} && -f $OPT{$k};
     }
-    output({color=>'warning',sticky=>1},
+    output({color=>'yellow',sticky=>1},
         sprintf "%s - warning ignoring %s as they are invalid in this context.", basename($0), join(', ', map { "--$_" } @ignored)
     ) if @ignored;
 } # End Mappings/Settings for Non-existant index.
@@ -174,13 +177,13 @@ while( $res && @{ $res->{hits}{hits} }) {
             body => $body
         );
         if ($s2 ne "200") {
-            output({stderr=>1,color=>'red'},"Failed to put documents to $OPT{to} (http status = $status): " . $JSON->encode([ $s2, $r2 ]));
+            output({stderr=>1,color=>'red'},"Failed to put documents to $HOST{to} (http status = $status): " . $JSON->encode([ $s2, $r2 ]));
             next;
         }
         $success=1;
         last;
     }
-    die "Failed to write data to $OPT{to}:9200/$INDEX{to} after $RECORDS docs indexed."
+    die "Failed to write data to $HOST{to}:9200/$INDEX{to} after $RECORDS docs indexed."
         unless $success;
     my $took = time - $start;
     show_counts( scalar @{$res->{hits}{hits}} );
@@ -191,16 +194,16 @@ while( $res && @{ $res->{hits}{hits} }) {
         }
     });
 
-    verbose(sprintf "Batch of %d done in %00.2fds.", $batch, $took);
+    verbose(sprintf "Batch of %d done in %00.2fs.", $batch, $took);
 }
 
 sub show_counts {
     my $inc_records = shift;
 
-    output({color=>'green'}, "Starting copy of $INDEX{from} to $OPT{to}:$INDEX{to}.") if $RECORDS == 0;
+    output({color=>'green'}, "Starting copy of $INDEX{from} to $HOST{to}:$INDEX{to}.") if $RECORDS == 0;
 
     $RECORDS += $inc_records;
-    if( $RECORDS % ($OPT{block} * 10) == 0 ) {
+    if( $RECORDS % ($INDEX{block} * 10) == 0 ) {
         my $now = time;
         my $diff = $now - $LAST;
         my @time=localtime;
@@ -222,7 +225,7 @@ es-copy-index.pl - Copy an index from one cluster to another
 
 =head1 VERSION
 
-version 4.3
+version 4.4
 
 =head1 SYNOPSIS
 
